/*****************************/
/* Van Gogh Oil Paint shader */
/*****************************/



/* check if current point is on back or front of object */
float point_on_back()
{

   extern normal N;
   extern vector I;
   float val=0;

   if(faceforward(N,I).N>=0.0) {
       
     val=0;  /* point is on front */
   }

   else {
    
     val=1;  /* point is on back */ 
   }

  return val;
}

/*-----------------------------------------------------------------------*/


/* glossy specular function */
color LocIllumGlossy (normal N;
                      vector V;
                      float roughness, sharpness;) 
{

    color C=0;

    float w=.18*(1-sharpness);

    extern point P;
    illuminance (P,N,PI/2) {
       
       extern vector L; 
       extern color Cl;
       float nonspec=0;
       lightsource("_nonspecular", nonspec);
       if(nonspec<1) {

          vector H=normalize(normalize(L)+V);
          C+=Cl*((1-nonspec)*
                smoothstep(.72-w, .72+w, pow(max(0,N.H), 1/roughness)));
       }
    }
    return C;
}

/*-----------------------------------------------------------------------*/


/* texture lookup function */
color readTexture(string name;
                  float ss, tt;)
{
   extern color Cs;
   color C;
   
   if(name!="") {
      C=Cs * color texture(name, ss, tt,"swidth",0,"twidth",0);  
   }
    else {
       C=Cs;
    }              
   return C;
}

/*-----------------------------------------------------------------------*/


/* oil paint material */
color MaterialOilPaint(normal Nf;
                       color texColour, specularcolour;
                       float Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination;)
{

   extern vector I;
   normal Nn=normalize(Nf);
   return texColour*(self_illumination +
                     Ka*ambient() + 
                      Kd*diffuse(Nn) +
                      Kt*diffuse(-Nn)) +
                      specularcolour*(Ks*LocIllumGlossy(Nn,
                                                        -normalize(I),
                                                        roughness,sharpness));

}

/*-----------------------------------------------------------------------*/


color getTransparency(float Hide_Back;
                      float ss,tt;
                      string name;) 
{
   
   float test=point_on_back();       /* check if point is on back or front of object */   
   color C=color texture(name, ss, tt,"filter","gaussian","swidth",1,"twidth",1);

   if(Hide_Back==1) {

      if(test==1) {
        C=0;
      }   
      else { C=color texture(name, ss, tt,"filter","gaussian","swidth",1,"twidth",1); }
   }
   else { C=color texture(name, ss, tt,"filter","gaussian","swidth",1,"twidth",1); }

  return C;
}

/*-----------------------------------------------------------------------*/


color assigncolour(string name;
                   float ss,tt;
                   color paintcolour;
                   float clamp;) 
{
   float grey,lightness,r,g,b;
   color final,paint;
  
   /* get grey value of image map - ranges from 0 to 1*/
   grey = float texture(name,ss,tt,"swidth",0,"twidth",0);

   /* resample grey value so that it falls between -0.5 and 0.5
   grey-=0.5;
   if(clamp==1) {
       if(grey<-0.35) { grey=-0.35; }
       if(grey>0.35) { grey=0.35; }
   }



   /* based on this grey value modify the paint colour */
   paint = paintcolour;

   r=comp(paint,0);
   g=comp(paint,1);
   b=comp(paint,2);

   r+=grey;
   g+=grey;
   b+=grey;

   setcomp(paint,0,r);
   setcomp(paint,1,g);
   setcomp(paint,2,b);

   return paint; 
 
}

/*-----------------------------------------------------------------------*/


surface VanGogh3 (float useUndercoat=1;
             float useShadows=1;
             float usePaint1=1;
             float usePaint2=1;
             float useHighlights=1;
             color undercoatColour=0.7;
             string undercoatColMap="";
             color shadowColour=0.1;
             string shadowColMap="";
             color paintColour1=0.4;
             string paint1ColMap="";
             color paintColour2=0.6;
             string paint2ColMap="";
             color highlightColour=0.8;
             string highlightColMap="";
             color specularcolour = 0.8;
             float usespecularmap = 1.;
             float clampColours=0;
             float Ka=.6, Kd=.6, Ks=.6, Kt=.725, roughness=.125, sharpness=.06, self_illumination=.1, self_illumination2=.1;
             float bumpscale = 0.025; 
             float dispscale = 0.015;
             float abDoDisplacement = 1;
             float Hide_Back=1;
             float UseOpacMap=0;
             point STMatrix0 = point "shader" (1,0,0);
             point STMatrix1 = point "shader" (0,1,0);
             point STTexMatrix0 = point "shader" (1,0,0);
             point STTexMatrix1 = point "shader" (0,1,0);)

{



    string undercoatMap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/undercoat"; 
    string shadowMap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/shadows"; 
    string stroke1Map = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/stroke1";
    string stroke2Map = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/stroke2";
    string highlightMap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/highlights"; 
    string transparencyMap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/transparency";
    string bumpmap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/bumpmap";
    string specularmap = "/tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/specularmap";
    float ss, tt, sss, ttt;                          /* texture attributes */
    float test=point_on_back();                      /* check if point is on back or front of object */
    color tex1,tex2,tex3,tex4,tex5,col;              /* texture colours - where texture data is stored*/
    vector Nn=normalize(N); 
    float alpha=0;   
    color spec = specularcolour;            
    
    /* /tmp_mnt/nfs/o2cg75/disk1/users/macvaf/u2rj7y/graphics/shaders/ */


    /* apply STMatrix to texture coordinates - ss and tt are used for positioning Van Gogh paint strokes*/
    ss = vector(s, t, 1) . vector(transform("shader", STMatrix0));
    tt = vector(s, t, 1) . vector(transform("shader", STMatrix1));

    /* apply STTexMatrix to texture coordinates - sss and ttt are used for positioning optional colour maps*/
    sss = vector(s, t, 1) . vector(transform("shader", STTexMatrix0));
    ttt = vector(s, t, 1) . vector(transform("shader", STTexMatrix1));



    /* read and apply the bumpmap - also apply displacement if applicable*/
    float amp=float texture(bumpmap[0],ss,tt,"filter","gaussian","swidth",1,"twidth",1);
    if((amp*bumpscale)>=0.01) { 

       if(abDoDisplacement==1) { P+=Nn*((amp*dispscale)/length(vtransform("shader",Nn))); }
       N=calculatenormal(P+((amp*bumpscale)*normalize(N)));
    }






    /* read and store brush stroke textures and then work out the colour that should be applied */
    /* by looking at appropriate colour variable */

    if(undercoatColMap=="") { 
     col=undercoatColour; 
     tex1=assigncolour(undercoatMap,ss,tt,col,clampColours); }
 
      else { 
        col=readTexture(undercoatColMap,sss,ttt); 
        tex1=col; }



    if(shadowColMap=="") { 
     col=shadowColour; 
     tex2=assigncolour(shadowMap,ss,tt,col,clampColours); }

      else { 
        col=readTexture(shadowColMap,sss,ttt); 
        tex2=col; }
      


    if(paint1ColMap=="") { 
     col=paintColour1; 
     tex3=assigncolour(stroke1Map,ss,tt,col,clampColours); }

      else { 
        col=readTexture(paint1ColMap,sss,ttt); 
        tex3=col; }  
      

    if(paint2ColMap=="") { 
     col=paintColour2; 
     tex4=assigncolour(stroke2Map,ss,tt,col,clampColours); }

      else { 
       col=readTexture(paint2ColMap,sss,ttt); 
       tex4=col; }
      

    if(highlightColMap=="") { 
     col=highlightColour; 
     tex5=assigncolour(highlightMap,ss,tt,col,clampColours);}

      else { 
       col=readTexture(highlightColMap,sss,ttt); 
       tex5=col; }
      









    /* calculate colour of surface at this point - layers of paint are applied one by one */
    normal Nf = faceforward(normalize(N),I);

    /* workout specular colour */
    if(usespecularmap==1) { spec=readTexture(specularmap,ss,tt); }
     else { spec=specularcolour; }


    if(Hide_Back==1) {

       if(test==0) {

           /* undercoat */
           if(useUndercoat==1) { 
             if(undercoatColMap=="") { Ci=MaterialOilPaint(Nf,tex1,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
              else { Ci=MaterialOilPaint(Nf,tex1,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
           }



           /* shadows */
           if(useShadows==1) {
               alpha=float texture(shadowMap,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(shadowColMap=="") {Ci=MaterialOilPaint(Nf,tex2,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else {Ci=MaterialOilPaint(Nf,tex2,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }



           /* stroke 1 */
           if(usePaint1==1) {
               alpha=float texture(stroke1Map,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(paint1ColMap=="") {Ci=MaterialOilPaint(Nf,tex3,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else {Ci=MaterialOilPaint(Nf,tex3,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }


           /* stroke 2 */
           if(usePaint2==1) {
              alpha=float texture(stroke2Map,ss,tt,"swidth",0,"twidth",0); 
              if(alpha<0.9) { 
                if(paint2ColMap=="") {Ci=MaterialOilPaint(Nf,tex4,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                  else {Ci=MaterialOilPaint(Nf,tex4,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
              }
	   }



           /* highlights */
           if(useHighlights==1) {
               alpha=float texture(highlightMap,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(highlightColMap=="") {Ci=MaterialOilPaint(Nf,tex5,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else { Ci=MaterialOilPaint(Nf,tex5,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }

       }
       else if(test==1) {
       
          Ci=0;
       }
    }
    else {

           /* undercoat */
           if(useUndercoat==1) { 
             if(undercoatColMap=="") { Ci=MaterialOilPaint(Nf,tex1,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
              else { Ci=MaterialOilPaint(Nf,tex1,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
           }



           /* shadows */
           if(useShadows==1) {
               alpha=float texture(shadowMap,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(shadowColMap=="") {Ci=MaterialOilPaint(Nf,tex2,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else {Ci=MaterialOilPaint(Nf,tex2,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }



           /* stroke 1 */
           if(usePaint1==1) {
               alpha=float texture(stroke1Map,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(paint1ColMap=="") {Ci=MaterialOilPaint(Nf,tex3,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else {Ci=MaterialOilPaint(Nf,tex3,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }


           /* stroke 2 */
           if(usePaint2==1) {
              alpha=float texture(stroke2Map,ss,tt,"swidth",0,"twidth",0); 
              if(alpha<0.9) { 
                if(paint2ColMap=="") {Ci=MaterialOilPaint(Nf,tex4,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                  else {Ci=MaterialOilPaint(Nf,tex4,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
              }
	   }



           /* highlights */
           if(useHighlights==1) {
               alpha=float texture(highlightMap,ss,tt,"swidth",0,"twidth",0); 
               if(alpha<0.9) { 
                 if(highlightColMap=="") {Ci=MaterialOilPaint(Nf,tex5,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination); }
                   else { Ci=MaterialOilPaint(Nf,tex5,spec,Ka,Kd,Ks,Kt,roughness,sharpness,self_illumination2); }
               }
           }

    }


    /* calculate transparency */
    if(UseOpacMap==1) { Oi=getTransparency(Hide_Back,ss,tt,transparencyMap); }
      else { Oi=Os; }

    Ci*=Oi;
}

















